@@ -321,6 +321,10 @@ vips_format_class_init( VipsFormatClass *class )
 	object_class->nickname = "format";
 	object_class->description = _( "VIPS file formats" );
 	object_class->summary_class = vips_format_summary_class;
+
+	/* Hide from UI.
+	 */
+	object_class->deprecated = TRUE;
 }
 
 static void
@@ -346,7 +350,7 @@ vips_format_get_flags( VipsFormatClass *format, const char *filename )
 /* VIPS format class.
  */
 
-static const char *vips_suffs[] = { ".v", NULL };
+static const char *vips_suffs[] = { ".v", ".vips", NULL };
 
 int
 im_isvips( const char *filename )
@@ -404,6 +408,12 @@ vips_flags( const char *filename )
 typedef VipsFormat VipsFormatVips;
 typedef VipsFormatClass VipsFormatVipsClass;
 
+static int
+vips_format_vips_save( VipsImage *image, const char *filename )
+{
+	return( vips_image_write_to_file( image, filename, NULL ) ); 
+}
+
 static void
 vips_format_vips_class_init( VipsFormatVipsClass *class )
 {
@@ -417,7 +427,7 @@ vips_format_vips_class_init( VipsFormatVipsClass *class )
 	format_class->is_a = im_isvips;
 	format_class->header = file2vips;
 	format_class->load = file2vips;
-	format_class->save = vips_image_write_to_file;
+	format_class->save = vips_format_vips_save;
 	format_class->get_flags = vips_flags;
 	format_class->suffs = vips_suffs;
 }
