@@ -43,6 +43,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <ctype.h>
 
 #include <vips/vips.h>
 #include <vips/internal.h>
@@ -50,6 +51,192 @@
 #include <vips/vector.h>
 #include <vips/transform.h>
 
+/* Split filename into name / mode components. name and mode should both be
+ * FILENAME_MAX chars.
+ *
+ * We look for the ':' splitting the name and mode by searching for the
+ * rightmost occurence of the regexp ".[A-Za-z0-9]+:". Example: consider the
+ * horror that is
+ *
+ * 	c:\silly:dir:name\fr:ed.tif:jpeg:95,,,,c:\icc\srgb.icc
+ *
+ */
+void
+im_filename_split( const char *path, char *name, char *mode )
+{
+        char *p;
+
+        vips_strncpy( name, path, FILENAME_MAX );
+
+	/* Search back towards start stopping at each ':' char.
+	 */
+	for( p = name + strlen( name ) - 1; p > name; p -= 1 )
+		if( *p == ':' ) {
+			char *q;
+
+			for( q = p - 1; isalnum( *q ) && q > name; q -= 1 )
+				;
+
+			if( *q == '.' )
+				break;
+		}
+
+	if( *p == ':' ) {
+                vips_strncpy( mode, p + 1, FILENAME_MAX );
+                *p = '\0';
+        }
+        else
+                strcpy( mode, "" );
+}
+
+/* Skip any leading path stuff. Horrible: if this is a filename which came
+ * from win32 and we're a *nix machine, it'll have '\\' not '/' as the
+ * separator :-(
+ *
+ * Try to fudge this ... if the file doesn't contain any of our native
+ * separators, look for the opposite one as well. If there are none of those
+ * either, just return the filename.
+ */
+const char *
+im_skip_dir( const char *path )
+{
+	char name[FILENAME_MAX];
+	char mode[FILENAME_MAX];
+        const char *p;
+        const char *q;
+
+	const char native_dir_sep = G_DIR_SEPARATOR;
+	const char non_native_dir_sep = native_dir_sep == '/' ? '\\' : '/';
+
+	/* Remove any trailing save modifiers: we don't want '/' or '\' in the
+	 * modifier confusing us.
+	 */
+	im_filename_split( path, name, mode );
+
+	/* The '\0' char at the end of the string.
+	 */
+	p = name + strlen( name );
+
+	/* Search back for the first native dir sep, or failing that, the first
+	 * non-native dir sep.
+	 */
+	for( q = p; q > name && q[-1] != native_dir_sep; q-- )
+		;
+	if( q == name )
+		for( q = p; q > name && q[-1] != non_native_dir_sep; q-- )
+			;
+
+        return( path + (q - name) );
+}
+
+/* Extract suffix from filename, ignoring any mode string. Suffix should be
+ * FILENAME_MAX chars. Include the "." character, if any.
+ */
+void
+im_filename_suffix( const char *path, char *suffix )
+{
+	char name[FILENAME_MAX];
+	char mode[FILENAME_MAX];
+        char *p;
+
+	im_filename_split( path, name, mode );
+        if( (p = strrchr( name, '.' )) ) 
+                strcpy( suffix, p );
+        else
+                strcpy( suffix, "" );
+}
+
+/* Does a filename have one of a set of suffixes. Ignore case.
+ */
+int
+im_filename_suffix_match( const char *path, const char *suffixes[] )
+{
+	char suffix[FILENAME_MAX];
+	const char **p;
+
+	im_filename_suffix( path, suffix );
+	for( p = suffixes; *p; p++ )
+		if( g_ascii_strcasecmp( suffix, *p ) == 0 )
+			return( 1 );
+
+	return( 0 );
+}
+
+/* p points to the start of a buffer ... move it on through the buffer (ready
+ * for the next call), and return the current option (or NULL for option
+ * missing). ',' characters inside options can be escaped with a '\'.
+ */
+char *
+im_getnextoption( char **in )
+{
+        char *p;
+        char *q;
+
+        p = *in;
+        q = p;
+
+        if( !p || !*p )
+                return( NULL );
+
+	/* Find the next ',' not prefixed with a '\'. If the first character
+	 * of p is ',', there can't be a previous escape character.
+	 */
+	for(;;) {
+		if( !(p = strchr( p, ',' )) )
+			break;
+		if( p == q )
+			break;
+		if( p[-1] != '\\' )
+			break;
+
+		p += 1;
+	}
+
+        if( p ) {
+                /* Another option follows this one .. set up to pick that out
+                 * next time.
+                 */
+                *p = '\0';
+                *in = p + 1;
+        }
+        else {
+                /* This is the last one.
+                 */
+                *in = NULL;
+        }
+
+        if( strlen( q ) > 0 )
+                return( q );
+        else
+                return( NULL );
+}
+
+/* Get a suboption string, or NULL.
+ */
+char *
+im_getsuboption( const char *buf )
+{
+        char *p, *q, *r;
+
+        if( !(p = strchr( buf, ':' )) ) 
+		/* No suboption.
+		 */
+		return( NULL );
+
+	/* Step over the ':'.
+	 */
+	p += 1;
+
+	/* Need to unescape any \, pairs. Shift stuff down one if we find one.
+	 */
+	for( q = p; *q; q++ ) 
+		if( q[0] == '\\' && q[1] == ',' )
+			for( r = q; *r; r++ )
+				r[0] = r[1];
+
+        return( p );
+}
+
 VipsImage *
 im_open( const char *filename, const char *mode )
 {
@@ -651,7 +838,7 @@ im__bandup( const char *domain, IMAGE *in, IMAGE *out, int n )
 	int i;
 
 	if( in->Bands == n ) 
-		return( im_copy( in, out ) );
+		return( vips_image_write( in, out ) );
 	if( in->Bands != 1 ) {
 		im_error( domain, _( "not one band or %d bands" ), n );
 		return( -1 );
@@ -845,7 +1032,7 @@ im_add( IMAGE *in1, IMAGE *in2, IMAGE *out )
 
 	if( vips_call( "add", in1, in2, &x, NULL ) )
 		return( -1 );
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -861,7 +1048,7 @@ im_subtract( IMAGE *in1, IMAGE *in2, IMAGE *out )
 
 	if( vips_call( "subtract", in1, in2, &x, NULL ) )
 		return( -1 );
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -877,7 +1064,7 @@ im_multiply( IMAGE *in1, IMAGE *in2, IMAGE *out )
 
 	if( vips_call( "multiply", in1, in2, &x, NULL ) )
 		return( -1 );
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -893,7 +1080,7 @@ im_divide( IMAGE *in1, IMAGE *in2, IMAGE *out )
 
 	if( vips_call( "divide", in1, in2, &x, NULL ) )
 		return( -1 );
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -1066,7 +1253,7 @@ im_copy_swap( IMAGE *in, IMAGE *out )
 int
 im_copy_set_meta( IMAGE *in, IMAGE *out, const char *field, GValue *value )
 {
-	if( im_copy( in, out ) )
+	if( vips_image_write( in, out ) )
 		return( -1 );
 	im_meta_set( out, field, value );
 
@@ -1079,7 +1266,7 @@ im_copy_native( IMAGE *in, IMAGE *out, gboolean is_msb_first )
 	if( is_msb_first != im_amiMSBfirst() )
 		return( im_copy_swap( in, out ) );
 	else
-		return( im_copy( in, out ) );
+		return( vips_image_write( in, out ) );
 }
 
 int
@@ -1754,14 +1941,24 @@ im_text( IMAGE *out, const char *text, const char *font,
 int 
 im_system( VipsImage *im, const char *cmd, char **out )
 {
+	VipsArea *area;
+	VipsImage **array;
 	char *str;
 
+	area = vips_area_new_array_object( 1 );
+	array = (VipsImage **) area->data;
+	array[0] = im;
+
 	if( vips_system( cmd, 
-		"in", im,
+		"in", area,
 		"in_format", "%s.v",
 		"log", &str,
-		NULL ) )
+		NULL ) ) {
+		vips_area_unref( area );
 		return( -1 );
+	}
+
+	vips_area_unref( area );
 
 	if( out )
 		*out = str;
@@ -1774,20 +1971,36 @@ im_system_image( VipsImage *im,
 	const char *in_format, const char *out_format, const char *cmd_format,
 	char **log )
 {
+	VipsArea *area;
+	VipsImage **array;
 	char *str;
 	VipsImage *out; 
 
+	area = vips_area_new_array_object( 1 );
+	array = (VipsImage **) area->data;
+	array[0] = im;
+
+	/* im will be unreffed when area is unreffed.
+	 */
+	g_object_ref( im ); 
+
 	if( vips_system( cmd_format, 
-		"in", im,
+		"in", area,
 		"out", &out,
 		"in_format", in_format,
 		"out_format", out_format,
 		"log", &str,
-		NULL ) )
+		NULL ) ) {
+		vips_area_unref( area );
 		return( NULL );
+	}
+
+	vips_area_unref( area );
 
 	if( log )
 		*log = str;
+	else
+		g_free( str ); 
 
 	return( out );
 }
@@ -2297,7 +2510,7 @@ im_spcor( IMAGE *in, IMAGE *ref, IMAGE *out )
 
 	if( vips_call( "spcor", in, ref, &x, NULL ) )
 		return( -1 );
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -2320,7 +2533,7 @@ im_fastcor( IMAGE *in, IMAGE *ref, IMAGE *out )
 
 	if( vips_call( "fastcor", in, ref, &x, NULL ) )
 		return( -1 );
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -2853,7 +3066,7 @@ im_ri2c( IMAGE *in1, IMAGE *in2, IMAGE *out )
 
 	if( vips_call( "complexform", in1, in2, &x, NULL ) )
 		return( -1 );
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -2875,7 +3088,7 @@ im_argb2rgba( VipsImage *in, VipsImage *out )
 {
 	/* No longer exists, just a null op.
 	 */
-	return( im_copy( in, out ) );
+	return( vips_image_write( in, out ) );
 }
 
 int
@@ -2885,7 +3098,7 @@ im_shrink( VipsImage *in, VipsImage *out, double xshrink, double yshrink )
 
 	if( vips_shrink( in, &x, xshrink, yshrink, NULL ) )
 		return( -1 );
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -2904,7 +3117,7 @@ im_eye( IMAGE *out, const int xsize, const int ysize, const double factor )
 		"uchar", TRUE,
 		NULL ) )
 		return( -1 );
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -2922,7 +3135,7 @@ im_feye( IMAGE *out, const int xsize, const int ysize, const double factor )
 		"factor", factor,
 		NULL ) )
 		return( -1 );
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -2940,7 +3153,7 @@ im_grey( IMAGE *out, const int xsize, const int ysize )
 		"uchar", TRUE,
 		NULL ) )
 		return( -1 );
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -2957,7 +3170,7 @@ im_fgrey( IMAGE *out, const int xsize, const int ysize )
 	if( vips_grey( &x, xsize, ysize, 
 		NULL ) )
 		return( -1 );
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -2981,7 +3194,7 @@ im_buildlut( DOUBLEMASK *input, VipsImage *out )
 		return( -1 );
 	}
 	g_object_unref( mat );
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -3006,7 +3219,7 @@ im_invertlut( DOUBLEMASK *input, VipsImage *out, int size )
 		return( -1 );
 	}
 	g_object_unref( mat );
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -3091,7 +3304,7 @@ im_Lab2XYZ_temp( IMAGE *in, IMAGE *out, double X0, double Y0, double Z0 )
 	}
 	vips_area_unref( temp );
 
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -3107,7 +3320,7 @@ im_Lab2XYZ( IMAGE *in, IMAGE *out )
 
 	if( vips_Lab2XYZ( in, &x, NULL ) )
 		return( -1 );
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -3133,7 +3346,7 @@ im_XYZ2Lab_temp( IMAGE *in, IMAGE *out, double X0, double Y0, double Z0 )
 	}
 	vips_area_unref( temp );
 
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -3149,7 +3362,7 @@ im_XYZ2Lab( IMAGE *in, IMAGE *out )
 
 	if( vips_XYZ2Lab( in, &x, NULL ) )
 		return( -1 );
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -3165,7 +3378,7 @@ im_Lab2LCh( IMAGE *in, IMAGE *out )
 
 	if( vips_Lab2LCh( in, &x, NULL ) )
 		return( -1 );
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -3181,7 +3394,7 @@ im_LCh2Lab( IMAGE *in, IMAGE *out )
 
 	if( vips_LCh2Lab( in, &x, NULL ) )
 		return( -1 );
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -3197,7 +3410,7 @@ im_LCh2UCS( IMAGE *in, IMAGE *out )
 
 	if( vips_LCh2CMC( in, &x, NULL ) )
 		return( -1 );
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -3213,7 +3426,7 @@ im_UCS2LCh( IMAGE *in, IMAGE *out )
 
 	if( vips_CMC2LCh( in, &x, NULL ) )
 		return( -1 );
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -3229,7 +3442,7 @@ im_XYZ2Yxy( IMAGE *in, IMAGE *out )
 
 	if( vips_XYZ2Yxy( in, &x, NULL ) )
 		return( -1 );
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -3245,7 +3458,7 @@ im_Yxy2XYZ( IMAGE *in, IMAGE *out )
 
 	if( vips_Yxy2XYZ( in, &x, NULL ) )
 		return( -1 );
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -3261,7 +3474,7 @@ im_float2rad( IMAGE *in, IMAGE *out )
 
 	if( vips_float2rad( in, &x, NULL ) )
 		return( -1 );
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -3277,7 +3490,7 @@ im_rad2float( IMAGE *in, IMAGE *out )
 
 	if( vips_rad2float( in, &x, NULL ) )
 		return( -1 );
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -3293,7 +3506,7 @@ im_Lab2LabQ( IMAGE *in, IMAGE *out )
 
 	if( vips_Lab2LabQ( in, &x, NULL ) )
 		return( -1 );
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -3309,7 +3522,7 @@ im_LabQ2Lab( IMAGE *in, IMAGE *out )
 
 	if( vips_LabQ2Lab( in, &x, NULL ) )
 		return( -1 );
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -3325,7 +3538,7 @@ im_Lab2LabS( IMAGE *in, IMAGE *out )
 
 	if( vips_Lab2LabS( in, &x, NULL ) )
 		return( -1 );
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -3341,7 +3554,7 @@ im_LabS2Lab( IMAGE *in, IMAGE *out )
 
 	if( vips_LabS2Lab( in, &x, NULL ) )
 		return( -1 );
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -3357,7 +3570,7 @@ im_LabQ2LabS( IMAGE *in, IMAGE *out )
 
 	if( vips_LabQ2LabS( in, &x, NULL ) )
 		return( -1 );
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -3373,7 +3586,7 @@ im_LabS2LabQ( IMAGE *in, IMAGE *out )
 
 	if( vips_LabS2LabQ( in, &x, NULL ) )
 		return( -1 );
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -3447,7 +3660,7 @@ im_sRGB2XYZ( IMAGE *in, IMAGE *out )
 
 	if( vips_sRGB2scRGB( in, &t[0], NULL ) ||
 		vips_scRGB2XYZ( t[0], &t[1], NULL ) ||
-		im_copy( t[1], out ) ) 
+		vips_image_write( t[1], out ) ) 
 		return( -1 );
 
 	return( 0 );
@@ -3461,7 +3674,7 @@ im_XYZ2sRGB( IMAGE *in, IMAGE *out )
 
 	if( vips_XYZ2scRGB( in, &t[0], NULL ) ||
 		vips_scRGB2sRGB( t[0], &t[1], NULL ) ||
-		im_copy( t[1], out ) ) 
+		vips_image_write( t[1], out ) ) 
 		return( -1 );
 
 	return( 0 );
@@ -3474,7 +3687,7 @@ im_LabQ2sRGB( IMAGE *in, IMAGE *out )
 
 	if( vips_LabQ2sRGB( in, &x, NULL ) )
 		return( -1 );
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -3496,7 +3709,7 @@ im_icc_transform( VipsImage *in, VipsImage *out,
 		"intent", intent,
 		NULL ) )
 		return( -1 );
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -3516,7 +3729,7 @@ im_icc_import( VipsImage *in, VipsImage *out,
 		"intent", intent,
 		NULL ) )
 		return( -1 );
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -3535,7 +3748,7 @@ im_icc_import_embedded( VipsImage *in, VipsImage *out, VipsIntent intent )
 		"intent", intent,
 		NULL ) )
 		return( -1 );
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -3556,7 +3769,7 @@ im_icc_export_depth( VipsImage *in, VipsImage *out, int depth,
 		"intent", intent,
 		NULL ) )
 		return( -1 );
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -3661,7 +3874,7 @@ im_dE_fromLab( IMAGE *in1, IMAGE *in2, IMAGE *out )
 	if( vips_dE76( in1, in2, &x, 
 		NULL ) )
 		return( -1 );
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -3678,7 +3891,7 @@ im_dECMC_fromLab( IMAGE *in1, IMAGE *in2, IMAGE *out )
 	if( vips_dECMC( in1, in2, &x, 
 		NULL ) )
 		return( -1 );
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -3695,7 +3908,7 @@ im_dE00_fromLab( IMAGE *in1, IMAGE *in2, IMAGE *out )
 	if( vips_dE00( in1, in2, &x, 
 		NULL ) )
 		return( -1 );
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -3711,7 +3924,7 @@ im_icc_ac2rc( VipsImage *in, VipsImage *out, const char *profile_filename )
 
 	if( vips_icc_ac2rc( in, &x, profile_filename ) )
 		return( -1 );
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -3728,7 +3941,7 @@ im_quadratic( IMAGE *in, IMAGE *out, IMAGE *coeff )
 	if( vips_quadratic( in, &x, coeff, 
 		NULL ) )
 		return( -1 );
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -3811,7 +4024,7 @@ im_cross_phase( IMAGE *in1, IMAGE *in2, IMAGE *out )
 	if( vips_call( "cross_phase", in1, in2, &x, NULL ) )
 		return( -1 );
 
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -3828,7 +4041,7 @@ im_maplut( IMAGE *in, IMAGE *out, IMAGE *lut )
 	if( vips_maplut( in, &x, lut, NULL ) )
 		return( -1 );
 
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -3858,7 +4071,7 @@ im_histcum( IMAGE *in, IMAGE *out )
 	if( vips_hist_cum( in, &x, NULL ) )
 		return( -1 );
 
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -3875,7 +4088,7 @@ im_histnorm( IMAGE *in, IMAGE *out )
 	if( vips_hist_norm( in, &x, NULL ) )
 		return( -1 );
 
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -3905,7 +4118,7 @@ im_heq( VipsImage *in, VipsImage *out, int bandno )
 	if( vips_hist_equal( in, &x, "band", bandno, NULL ) )
 		return( -1 );
 
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -3937,7 +4150,7 @@ im_histgr( IMAGE *in, IMAGE *out, int bandno )
 		NULL ) )
 		return( -1 );
 
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -3961,7 +4174,7 @@ im_stdif( IMAGE *in, IMAGE *out,
 		NULL ) )
 		return( -1 );
 
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -3978,7 +4191,7 @@ im_lhisteq( VipsImage *in, VipsImage *out, int width, int height )
 	if( vips_hist_local( in, &x, width, height, NULL ) )
 		return( -1 );
 
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -3997,7 +4210,7 @@ im_histnD( VipsImage *in, VipsImage *out, int bins )
 		NULL ) )
 		return( -1 );
 
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -4014,7 +4227,7 @@ im_hist_indexed( VipsImage *index, VipsImage *value, VipsImage *out )
 	if( vips_hist_find_indexed( value, index, &x, NULL ) )
 		return( -1 );
 
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -4031,14 +4244,14 @@ im_project( IMAGE *in, IMAGE *hout, IMAGE *vout )
 	if( vips_project( in, &x, &y, NULL ) )
 		return( -1 );
 
-	if( im_copy( x, hout ) ) {
+	if( vips_image_write( x, hout ) ) {
 		g_object_unref( x );
 		g_object_unref( y );
 		return( -1 );
 	}
 	g_object_unref( x );
 
-	if( im_copy( y, vout ) ) {
+	if( vips_image_write( y, vout ) ) {
 		g_object_unref( y );
 		return( -1 );
 	}
@@ -4070,7 +4283,7 @@ im_profile( IMAGE *in, IMAGE *out, int dir )
 	}
 	g_object_unref( t1 );
 
-	if( im_copy( t2, out ) ) {
+	if( vips_image_write( t2, out ) ) {
 		g_object_unref( t2 );
 		return( -1 );
 	}
@@ -4121,7 +4334,7 @@ match( VipsImage *in, VipsImage *ref, VipsImage *out )
 	if( vips_hist_match( in, ref, &x, NULL ) )
 		return( -1 );
 
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -4169,7 +4382,7 @@ im_falsecolour( IMAGE *in, IMAGE *out )
 	if( vips_falsecolour( in, &x, NULL ) )
 		return( -1 );
 
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -4188,7 +4401,7 @@ im_gammacorrect( IMAGE *in, IMAGE *out, double exponent )
 		NULL ) )
 		return( -1 );
 
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -4228,7 +4441,35 @@ im_tile_cache( IMAGE *in, IMAGE *out,
 		NULL ) )
 		return( -1 );
 
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
+		g_object_unref( x );
+		return( -1 );
+	}
+	g_object_unref( x );
+
+	return( 0 );
+}
+
+/* This is the one used by nip2's menu for caching images. Random access and
+ * persistent. 
+ */
+int
+im_tile_cache_random( IMAGE *in, IMAGE *out,
+	int tile_width, int tile_height, int max_tiles )
+{
+	VipsImage *x;
+
+	if( vips_tilecache( in, &x, 
+		"tile_width", tile_width, 
+		"tile_height", tile_height, 
+		"max_tiles", max_tiles, 
+		"access", VIPS_ACCESS_RANDOM,
+		"persistent", TRUE,
+		"threaded", TRUE, 
+		NULL ) )
+		return( -1 );
+
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -4351,7 +4592,7 @@ im_copy_file( IMAGE *in, IMAGE *out )
 
 	if( vips_copy_file( in, &x, NULL ) )
 		return( -1 );
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -4378,7 +4619,7 @@ im_fwfft( IMAGE *in, IMAGE *out )
 	if( vips_fwfft( in, &x, NULL ) )
 		return( -1 );
 
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -4395,7 +4636,7 @@ im_invfft( IMAGE *in, IMAGE *out )
 	if( vips_invfft( in, &x, NULL ) )
 		return( -1 );
 
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -4414,7 +4655,7 @@ im_invfftr( IMAGE *in, IMAGE *out )
 		NULL ) )
 		return( -1 );
 
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -4431,7 +4672,7 @@ im_freqflt( IMAGE *in, IMAGE *mask, IMAGE *out )
 	if( vips_freqmult( in, mask, &x, NULL ) )
 		return( -1 );
 
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -4480,7 +4721,7 @@ im_phasecor_fft( IMAGE *in1, IMAGE *in2, IMAGE *out )
 	if( vips_phasecor( in1, in2, &x, NULL ) )
 		return( -1 );
 
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -4498,6 +4739,23 @@ im_cntlines( VipsImage *im, double *nolines, int flag )
 		NULL ) );
 }
 
+int
+im_label_regions( IMAGE *test, IMAGE *mask, int *segments )
+{
+	VipsImage *x;
+
+	if( vips_labelregions( test, &x, "segments", segments, NULL ) )
+		return( -1 );
+
+	if( vips_image_write( x, mask ) ) {
+		g_object_unref( x );
+		return( -1 );
+	}
+	g_object_unref( x );
+
+	return( 0 );
+}
+
 int 
 im_rank( IMAGE *in, IMAGE *out, int width, int height, int index )
 {
@@ -4506,7 +4764,7 @@ im_rank( IMAGE *in, IMAGE *out, int width, int height, int index )
 	if( vips_rank( in, &x, width, height, index, NULL ) )
 		return( -1 );
 
-	if( im_copy( x, out ) ) {
+	if( vips_image_write( x, out ) ) {
 		g_object_unref( x );
 		return( -1 );
 	}
@@ -4521,3 +4779,477 @@ im_rank_raw( IMAGE *in, IMAGE *out, int width, int height, int index )
 	im_error( "im_rank_raw", "no compat function" );
 	return( -1 );
 }
+
+int
+im_draw_circle( VipsImage *image, 
+	int x, int y, int radius, gboolean fill, VipsPel *ink )
+{
+	double *vec;
+	int n;
+
+	if( !(vec = vips__ink_to_vector( "im_draw_circle", image, ink, &n )) )
+		return( -1 ); 
+
+	return( vips_draw_circle( image, vec, n, x, y, radius,
+		"fill", fill,
+		NULL ) ); 
+}
+
+int 
+im_draw_line( VipsImage *image, int x1, int y1, int x2, int y2, VipsPel *ink )
+{
+	double *vec;
+	int n;
+
+	if( !(vec = vips__ink_to_vector( "im_draw_line", image, ink, &n )) )
+		return( -1 ); 
+
+	return( vips_draw_line( image, vec, n, x1, y1, x2, y2, NULL ) ); 
+}
+
+typedef struct _Line {
+	VipsPlotFn plot;
+	void *a;
+	void *b;
+	void *c;
+} Line;
+
+static void
+draw_line_wrapper( VipsImage *image, int x, int y, void *client )
+{
+	Line *line = (Line *) client;
+
+	line->plot( image, x, y, line->a, line->b, line->c ); 
+}
+
+int 
+im_draw_line_user( VipsImage *image, 
+	int x1, int y1, int x2, int y2, 
+	VipsPlotFn plot, void *a, void *b, void *c )
+{
+	Line line;
+
+	line.plot = plot;
+	line.a = a;
+	line.b = b;
+	line.c = c;
+
+	vips__draw_line_direct( image, x1, y1, x2, y2, 
+		draw_line_wrapper, &line ); 
+	
+	return( 0 );
+}
+
+int
+im_draw_mask( VipsImage *image, VipsImage *mask, int x, int y, VipsPel *ink )
+{
+	/* Direct call, since this has to be very quick.
+	 */
+	return( vips__draw_mask_direct( image, mask, ink, x, y ) ); 
+}
+
+int
+im_draw_image( VipsImage *image, VipsImage *sub, int x, int y )
+{
+	return( vips_draw_image( image, sub, x, y, NULL ) ); 
+}
+
+int
+im_draw_rect( IMAGE *image, 
+	int left, int top, int width, int height, int fill, VipsPel *ink )
+{
+	double *vec;
+	int n;
+
+	if( !(vec = vips__ink_to_vector( "im_draw_rect", image, ink, &n )) )
+		return( -1 ); 
+
+	return( vips_draw_rect( image, vec, n, left, top, width, height,
+		"fill", fill, 
+		NULL ) ); 
+}
+
+int
+im_draw_point( VipsImage *image, int x, int y, VipsPel *ink )
+{
+	double *vec;
+	int n;
+
+	if( !(vec = vips__ink_to_vector( "im_draw_rect", image, ink, &n )) )
+		return( -1 ); 
+
+	return( vips_draw_point( image, vec, n, x, y, NULL ) ); 
+}
+
+int
+im_draw_smudge( VipsImage *im, int left, int top, int width, int height )
+{
+	return( vips_draw_smudge( im, left, top, width, height, NULL ) ); 
+}
+
+int
+im_read_point( VipsImage *image, int x, int y, VipsPel *ink )
+{
+	double *vector;
+	int n;
+	VipsPel *pixel_vector;
+
+	if( vips_getpoint( image, &vector, &n, x, y, NULL ) )
+		return( -1 );
+
+	if( !(pixel_vector = vips__vector_to_ink( "im_read_point", 
+		image, vector, NULL, n )) ) {
+		g_free( vector );
+		return( -1 );
+	}
+
+	memcpy( ink, pixel_vector, VIPS_IMAGE_SIZEOF_PEL( image ) ); 
+
+	g_free( vector );
+
+	return( 0 );
+}
+
+int
+im_draw_flood( IMAGE *image, int x, int y, VipsPel *ink, Rect *dout )
+{
+	double *vec;
+	int n;
+	int left;
+	int top;
+	int width;
+	int height;
+
+	if( !(vec = vips__ink_to_vector( "im_draw_flood", image, ink, &n )) )
+		return( -1 ); 
+
+	if( vips_draw_flood( image, vec, n, x, y,
+		"left", &left,
+		"top", &top,
+		"width", &width,
+		"height", &height,
+		NULL ) )
+		return( -1 ); 
+
+	if( dout ) { 
+		dout->left = left; 
+		dout->top = top; 
+		dout->width = width; 
+		dout->height = height; 
+	}
+
+	return( 0 ); 
+}
+
+int
+im_draw_flood_blob( IMAGE *image, int x, int y, VipsPel *ink, Rect *dout )
+{
+	double *vec;
+	int n;
+	int left;
+	int top;
+	int width;
+	int height;
+
+	if( !(vec = vips__ink_to_vector( "im_draw_flood", image, ink, &n )) )
+		return( -1 ); 
+
+	if( vips_draw_flood( image, vec, n, x, y,
+		"equal", TRUE,
+		"left", &left,
+		"top", &top,
+		"width", &width,
+		"height", &height,
+		NULL ) )
+		return( -1 ); 
+
+	if( dout ) { 
+		dout->left = left; 
+		dout->top = top; 
+		dout->width = width; 
+		dout->height = height; 
+	}
+
+	return( 0 ); 
+}
+
+int
+im_draw_flood_other( IMAGE *image, 
+	IMAGE *test, int x, int y, int serial, Rect *dout )
+{
+	int left;
+	int top;
+	int width;
+	int height;
+
+	if( vips_draw_flood1( image, serial, x, y,
+		"test", test,
+		"equal", TRUE,
+		"left", &left,
+		"top", &top,
+		"width", &width,
+		"height", &height,
+		NULL ) )
+		return( -1 ); 
+
+	if( dout ) { 
+		dout->left = left; 
+		dout->top = top; 
+		dout->width = width; 
+		dout->height = height; 
+	} 
+
+	return( 0 ); 
+}
+
+int
+im_lineset( IMAGE *in, IMAGE *out, IMAGE *mask, IMAGE *ink,
+	int n, int *x1v, int *y1v, int *x2v, int *y2v )
+{
+	Rect mask_rect;
+	int i;
+
+	if( mask->Bands != 1 || mask->BandFmt != IM_BANDFMT_UCHAR ||
+		mask->Coding != IM_CODING_NONE ) {
+		im_error( "im_lineset", 
+			"%s", _( "mask image not 1 band 8 bit uncoded" ) );
+		return( -1 );
+	}
+	if( ink->Bands != in->Bands || ink->BandFmt != in->BandFmt ||
+		ink->Coding != in->Coding ) {
+		im_error( "im_lineset", 
+			"%s", _( "ink image does not match in image" ) );
+		return( -1 );
+	}
+	if( ink->Xsize != 1 || ink->Ysize != 1 ) {
+		im_error( "im_lineset", "%s", _( "ink image not 1x1 pixels" ) );
+		return( -1 );
+	}
+
+	/* Copy the image then fastline to it ... this will render to a "t"
+	 * usually.
+	 */
+	if( vips_image_write( in, out ) )
+		return( -1 );
+
+	mask_rect.left = mask->Xsize / 2;
+	mask_rect.top = mask->Ysize / 2;
+	mask_rect.width = mask->Xsize;
+	mask_rect.height = mask->Ysize;
+
+	if( im_incheck( ink ) ||
+		im_incheck( mask ) )
+		return( -1 );
+
+	for( i = 0; i < n; i++ ) {
+		if( im_fastlineuser( out, x1v[i], y1v[i], x2v[i], y2v[i], 
+			im_plotmask, ink->data, mask->data, &mask_rect ) )
+			return( -1 );
+	}
+
+	return( 0 );
+}
+
+int
+im_match_linear_search( IMAGE *ref, IMAGE *sec, IMAGE *out,
+	int xr1, int yr1, int xs1, int ys1, 
+	int xr2, int yr2, int xs2, int ys2,
+	int hwindowsize, int hsearchsize )
+{
+	VipsImage *x;
+
+	if( vips_match( ref, sec, &x,
+		xr1, yr1, xs1, ys1, xr2, yr2, xs2, ys2,
+		"search", TRUE,
+		"hwindow", hwindowsize,
+		"harea", hsearchsize,
+		NULL ) )
+		return( -1 );
+
+	if( vips_image_write( x, out ) ) {
+		g_object_unref( x );
+		return( -1 );
+	}
+	g_object_unref( x );
+
+	return( 0 );
+}
+
+int
+im_match_linear( IMAGE *ref, IMAGE *sec, IMAGE *out,
+	int xr1, int yr1, int xs1, int ys1, 
+	int xr2, int yr2, int xs2, int ys2 )
+{ 
+	VipsImage *x;
+
+	if( vips_match( ref, sec, &x,
+		xr1, yr1, xs1, ys1, xr2, yr2, xs2, ys2,
+		NULL ) )
+		return( -1 );
+
+	if( vips_image_write( x, out ) ) {
+		g_object_unref( x );
+		return( -1 );
+	}
+	g_object_unref( x );
+
+	return( 0 );
+}
+
+int
+im_global_balance( IMAGE *in, IMAGE *out, double gamma )
+{
+	VipsImage *x;
+
+	if( vips_globalbalance( in, &x,
+		"gamma", gamma,
+		"int_output", TRUE,
+		NULL ) )
+		return( -1 );
+
+	if( vips_image_write( x, out ) ) {
+		g_object_unref( x );
+		return( -1 );
+	}
+	g_object_unref( x );
+
+	return( 0 );
+}
+
+int
+im_global_balancef( IMAGE *in, IMAGE *out, double gamma )
+{
+	VipsImage *x;
+
+	if( vips_globalbalance( in, &x,
+		"gamma", gamma,
+		NULL ) )
+		return( -1 );
+
+	if( vips_image_write( x, out ) ) {
+		g_object_unref( x );
+		return( -1 );
+	}
+	g_object_unref( x );
+
+	return( 0 );
+}
+
+int
+im_remosaic( IMAGE *in, IMAGE *out, const char *old_str, const char *new_str )
+{
+	VipsImage *x;
+
+	if( vips_remosaic( in, &x, old_str, new_str, NULL ) ) 
+		return( -1 );
+
+	if( vips_image_write( x, out ) ) {
+		g_object_unref( x );
+		return( -1 );
+	}
+	g_object_unref( x );
+
+	return( 0 );
+}
+
+int
+im_lrmosaic1( IMAGE *ref, IMAGE *sec, IMAGE *out, 
+	int bandno,
+	int xr1, int yr1, int xs1, int ys1, 
+	int xr2, int yr2, int xs2, int ys2,
+	int hwindowsize, int hsearchsize,
+	int balancetype,
+	int mwidth )
+{ 
+	VipsImage *x;
+
+	if( vips_mosaic1( ref, sec, &x, VIPS_DIRECTION_HORIZONTAL,
+		xr1, yr1, xs1, ys1, xr2, yr2, xs2, ys2,
+		"search", TRUE,
+		"bandno", bandno,
+		"hwindow", hwindowsize,
+		"harea", hsearchsize,
+		"mblend", mwidth,
+		NULL ) ) 
+		return( -1 );
+	if( vips_image_write( x, out ) ) {
+		g_object_unref( x );
+		return( -1 );
+	}
+	g_object_unref( x );
+
+	return( 0 );
+}
+
+int
+im_tbmosaic1( IMAGE *ref, IMAGE *sec, IMAGE *out,
+	int bandno,
+	int xr1, int yr1, int xs1, int ys1, 
+	int xr2, int yr2, int xs2, int ys2,
+	int hwindowsize, int hsearchsize,
+	int balancetype,
+	int mwidth )
+{ 
+	VipsImage *x;
+
+	if( vips_mosaic1( ref, sec, &x, VIPS_DIRECTION_VERTICAL,
+		xr1, yr1, xs1, ys1, xr2, yr2, xs2, ys2,
+		"search", TRUE,
+		"bandno", bandno,
+		"hwindow", hwindowsize,
+		"harea", hsearchsize,
+		"mblend", mwidth,
+		NULL ) ) 
+		return( -1 );
+	if( vips_image_write( x, out ) ) {
+		g_object_unref( x );
+		return( -1 );
+	}
+	g_object_unref( x );
+
+	return( 0 );
+}
+
+int
+im_lrmerge1( IMAGE *ref, IMAGE *sec, IMAGE *out,
+	int xr1, int yr1, int xs1, int ys1, 
+	int xr2, int yr2, int xs2, int ys2,
+	int mwidth )
+{
+	VipsImage *x;
+
+	if( vips_mosaic1( ref, sec, &x, VIPS_DIRECTION_HORIZONTAL,
+		xr1, yr1, xs1, ys1, xr2, yr2, xs2, ys2,
+		"mblend", mwidth,
+		NULL ) ) 
+		return( -1 );
+	if( vips_image_write( x, out ) ) {
+		g_object_unref( x );
+		return( -1 );
+	}
+	g_object_unref( x );
+
+	return( 0 );
+}
+
+int
+im_tbmerge1( IMAGE *ref, IMAGE *sec, IMAGE *out,
+	int xr1, int yr1, int xs1, int ys1, 
+	int xr2, int yr2, int xs2, int ys2,
+	int mwidth )
+{
+	VipsImage *x;
+
+	if( vips_mosaic1( ref, sec, &x, VIPS_DIRECTION_VERTICAL,
+		xr1, yr1, xs1, ys1, xr2, yr2, xs2, ys2,
+		"mblend", mwidth,
+		NULL ) ) 
+		return( -1 );
+	if( vips_image_write( x, out ) ) {
+		g_object_unref( x );
+		return( -1 );
+	}
+	g_object_unref( x );
+
+	return( 0 );
+}
