@@ -112,6 +112,8 @@ vips_foreign_load_openslide_header( VipsForeignLoad *load )
 		openslide->level, openslide->associated ) )
 		return( -1 );
 
+	VIPS_SETSTR( load->out->filename, openslide->filename );
+
 	return( 0 );
 }
 
@@ -140,7 +142,9 @@ static const char *vips_foreign_openslide_suffs[] = {
 	".vms", ".vmu", ".ndpi",  /* Hamamatsu */
 	".scn",		/* Leica */
 	".mrxs", 	/* MIRAX */
+	".svslide",	/* Sakura */
 	".tif", 	/* Trestle */
+	".bif", 	/* Ventana */
 	NULL
 };
 
