@@ -143,6 +143,14 @@ int vips__write_header_bytes( VipsImage *im, unsigned char *to );
 
 extern GMutex *vips__global_lock;
 
+int vips_image_written( VipsImage *image );
+void vips_image_preeval( VipsImage *image );
+void vips_image_eval( VipsImage *image, guint64 processed );
+void vips_image_posteval( VipsImage *image );
+gboolean vips_image_iskilled( VipsImage *image );
+void vips_image_set_kill( VipsImage *image, gboolean kill );
+VipsImage *vips_image_new_mode( const char *filename, const char *mode );
+
 int vips__formatalike_vec( VipsImage **in, VipsImage **out, int n );
 int vips__sizealike_vec( VipsImage **in, VipsImage **out, int n );
 int vips__bandup( const char *domain, VipsImage *in, VipsImage **out, int n );
@@ -259,15 +267,30 @@ int *im_offsets45( int size );
 int im_conv_f_raw( VipsImage *in, VipsImage *out, DOUBLEMASK *mask );
 int im_convsep_f_raw( VipsImage *in, VipsImage *out, DOUBLEMASK *mask );
 
-/* inplace
+/* draw
  */
-
 VipsPel *vips__vector_to_ink( const char *domain, 
-	VipsImage *im, double *vec, int n );
+	VipsImage *im, double *real, double *imag, int n );
+double *vips__ink_to_vector( const char *domain, 
+	VipsImage *im, VipsPel *ink, int *n ); 
+
 VipsPel *im__vector_to_ink( const char *domain, 
 	VipsImage *im, int n, double *vec );
-VipsImage *im__inplace_base( const char *domain, 
-	VipsImage *main, VipsImage *sub, VipsImage *out );
+
+int vips__draw_flood_direct( VipsImage *image, VipsImage *test, 
+	int serial, int x, int y );
+int vips__draw_mask_direct( VipsImage *image, VipsImage *mask, 
+	VipsPel *ink, int x, int y ); 
+
+typedef void (*VipsDrawPoint)( VipsImage *image, 
+	int x, int y, void *client ); 
+typedef void (*VipsDrawScanline)( VipsImage *image, 
+	int y, int x1, int x2, void *client );
+
+void vips__draw_line_direct( VipsImage *image, int x1, int y1, int x2, int y2,
+	VipsDrawPoint draw_point, void *client );
+void vips__draw_circle_direct( VipsImage *image, int cx, int cy, int r,
+	VipsDrawScanline draw_scanline, void *client );
 
 /* Register base vips interpolators, called during startup.
  */
@@ -289,14 +312,22 @@ void vips_freqfilt_operation_init( void );
 void vips_create_operation_init( void );
 void vips_morphology_operation_init( void );
 void vips_convolution_operation_init( void );
+void vips_draw_operation_init( void );
+void vips_mosaicing_operation_init( void );
 
 guint64 vips__parse_size( const char *size_string );
+int vips__substitute( const char *domain, char *buf, size_t len, char *sub );
 
 IMAGE *vips__deprecated_open_read( const char *filename, gboolean sequential );
 IMAGE *vips__deprecated_open_write( const char *filename );
 
 int vips__input_interpolate_init( im_object *obj, char *str );
 
+int vips_foreign_load( const char *filename, VipsImage **out, ... )
+	__attribute__((sentinel));
+int vips_foreign_save( VipsImage *in, const char *filename, ... )
+	__attribute__((sentinel));
+
 #ifdef __cplusplus
 }
 #endif /*__cplusplus*/
