@@ -95,6 +95,14 @@ extern "C" {
 #  endif
 #endif /*SWIG*/
 
+/* Or if this isn't gcc.
+ */
+#ifndef __GNUC__
+#  ifndef __attribute__
+#    define __attribute__(x)  /*NOTHING*/
+#  endif
+#endif /*__GNUC__*/
+
 #include <vips/basic.h>
 
 #include <vips/buf.h>
@@ -134,7 +142,7 @@ extern "C" {
 #include <vips/freqfilt.h>
 #include <vips/resample.h>
 #include <vips/colour.h>
-#include <vips/inplace.h>
+#include <vips/draw.h>
 #include <vips/create.h>
 #include <vips/video.h>
 #include <vips/cimg_funcs.h>
