@@ -548,12 +548,6 @@ size_t im_ref_string_get_length( const GValue *value );
 #define im_break_token vips_break_token
 #define im_vsnprintf vips_vsnprintf
 #define im_snprintf vips_snprintf
-#define im_filename_split vips_filename_split
-#define im_skip_dir vips_skip_dir
-#define im_filename_suffix vips_filename_suffix
-#define im_filename_suffix_match vips_filename_suffix_match
-#define im_getnextoption vips_getnextoption
-#define im_getsuboption vips_getsuboption
 #define im_file_length vips_file_length
 #define im__write vips__write
 #define im__file_open_read vips__file_open_read
@@ -783,6 +777,9 @@ int im_argb2rgba( VipsImage *in, VipsImage *out );
 int im_falsecolour( VipsImage *in, VipsImage *out );
 int im_gammacorrect( VipsImage *in, VipsImage *out, double exponent );
 
+int im_tile_cache_random( IMAGE *in, IMAGE *out,
+	int tile_width, int tile_height, int max_tiles );
+
 int im_shrink( VipsImage *in, VipsImage *out, double xshrink, double yshrink );
 int im_affinei( VipsImage *in, VipsImage *out, 
 	VipsInterpolate *interpolate,
@@ -998,9 +995,102 @@ int im_fractsurf( VipsImage *out, int size, double frd );
 int im_phasecor_fft( VipsImage *in1, VipsImage *in2, VipsImage *out );
 
 int im_cntlines( VipsImage *im, double *nolines, int flag );
+int im_label_regions( VipsImage *test, VipsImage *mask, int *segments );
 int im_rank( VipsImage *in, VipsImage *out, int width, int height, int index );
 int im_zerox( VipsImage *in, VipsImage *out, int sign );
 
+int im_draw_circle( VipsImage *image, 
+	int x, int y, int radius, gboolean fill, VipsPel *ink );
+
+int im_draw_mask( VipsImage *image, 
+	VipsImage *mask_im, int x, int y, VipsPel *ink );
+int im_draw_image( VipsImage *image, VipsImage *sub, int x, int y );
+int im_draw_rect( VipsImage *image, 
+	int left, int top, int width, int height, int fill, VipsPel *ink );
+
+int im_draw_line_user( VipsImage *image, 
+	int x1, int y1, int x2, int y2, 
+	VipsPlotFn plot, void *a, void *b, void *c );
+int im_draw_line( VipsImage *image, 
+	int x1, int y1, int x2, int y2, VipsPel *ink );
+int im_lineset( VipsImage *in, VipsImage *out, VipsImage *mask, VipsImage *ink,
+	int n, int *x1v, int *y1v, int *x2v, int *y2v );
+
+int im_draw_flood( VipsImage *image, int x, int y, VipsPel *ink, VipsRect *dout );
+int im_draw_flood_blob( VipsImage *image, 
+	int x, int y, VipsPel *ink, VipsRect *dout );
+int im_draw_flood_other( VipsImage *image, VipsImage *test, 
+	int x, int y, int serial, VipsRect *dout );
+
+int im_draw_point( VipsImage *image, int x, int y, VipsPel *ink );
+int im_read_point( VipsImage *image, int x, int y, VipsPel *ink );
+
+int im_draw_smudge( VipsImage *image, 
+	int left, int top, int width, int height );
+
+void im_filename_split( const char *path, char *name, char *mode );
+const char *im_skip_dir( const char *filename );
+void im_filename_suffix( const char *path, char *suffix );
+int im_filename_suffix_match( const char *path, const char *suffixes[] );
+char *im_getnextoption( char **in );
+char *im_getsuboption( const char *buf );
+
+int im_lrmerge( VipsImage *ref, VipsImage *sec, VipsImage *out,
+	int dx, int dy, int mwidth );
+int im_tbmerge( VipsImage *ref, VipsImage *sec, VipsImage *out,
+	int dx, int dy, int mwidth );
+
+int im_lrmosaic( VipsImage *ref, VipsImage *sec, VipsImage *out,
+	int bandno,
+	int xref, int yref, int xsec, int ysec,
+	int hwindowsize, int hsearchsize,
+	int balancetype,
+	int mwidth );
+int im_tbmosaic( VipsImage *ref, VipsImage *sec, VipsImage *out, 
+	int bandno,
+	int xref, int yref, int xsec, int ysec, 
+	int hwindowsize, int hsearchsize,
+	int balancetype,
+	int mwidth );
+
+int im_match_linear( VipsImage *ref, VipsImage *sec, VipsImage *out,
+	int xr1, int yr1, int xs1, int ys1,
+	int xr2, int yr2, int xs2, int ys2 );
+int im_match_linear_search( VipsImage *ref, VipsImage *sec, VipsImage *out,
+	int xr1, int yr1, int xs1, int ys1,
+	int xr2, int yr2, int xs2, int ys2,
+	int hwindowsize, int hsearchsize );
+
+int im_global_balance( VipsImage *in, VipsImage *out, double gamma );
+int im_global_balancef( VipsImage *in, VipsImage *out, double gamma );
+
+int im_remosaic( VipsImage *in, VipsImage *out,
+	const char *old_str, const char *new_str );
+
+int im_lrmerge1( VipsImage *ref, VipsImage *sec, VipsImage *out,
+	int xr1, int yr1, int xs1, int ys1,
+	int xr2, int yr2, int xs2, int ys2,
+	int mwidth );
+int im_tbmerge1( VipsImage *ref, VipsImage *sec, VipsImage *out,
+	int xr1, int yr1, int xs1, int ys1,
+	int xr2, int yr2, int xs2, int ys2,
+	int mwidth );
+
+int im_lrmosaic1( VipsImage *ref, VipsImage *sec, VipsImage *out, 
+	int bandno,
+	int xr1, int yr1, int xs1, int ys1, 
+	int xr2, int yr2, int xs2, int ys2,
+	int hwindowsize, int hsearchsize,
+	int balancetype,
+	int mwidth );
+int im_tbmosaic1( VipsImage *ref, VipsImage *sec, VipsImage *out,
+	int bandno,
+	int xr1, int yr1, int xs1, int ys1,
+	int xr2, int yr2, int xs2, int ys2,
+	int hwindowsize, int hsearchsize,
+	int balancetype,
+	int mwidth );
+
 #ifdef __cplusplus
 }
 #endif /*__cplusplus*/
