@@ -76,6 +76,10 @@
 #include <limits.h>
 #include <string.h>
 
+#ifdef HAVE_GSF
+#include <gsf/gsf.h>
+#endif /*HAVE_GSF*/
+
 #include <vips/vips.h>
 #include <vips/thread.h>
 #include <vips/internal.h>
@@ -244,7 +248,8 @@ vips__init( const char *argv0 )
 
 	/* Default info setting from env.
 	 */
-	if( g_getenv( "IM_INFO" ) ) 
+	if( g_getenv( "VIPS_INFO" ) || 
+		g_getenv( "IM_INFO" ) ) 
 		vips__info = 1;
 
 	/* Register base vips types.
@@ -272,6 +277,8 @@ vips__init( const char *argv0 )
 	vips_convolution_operation_init();
 	vips_freqfilt_operation_init();
 	vips_morphology_operation_init();
+	vips_draw_operation_init();
+	vips_mosaicing_operation_init();
 
 	/* Load up any plugins in the vips libdir. We don't error on failure,
 	 * it's too annoying to have VIPS refuse to start because of a broken
@@ -303,6 +310,12 @@ vips__init( const char *argv0 )
 	 */
 	vips_vector_init();
 
+#ifdef HAVE_GSF
+	/* Use this for structured file write.
+	 */
+	gsf_init();
+#endif /*HAVE_GSF*/
+
 	/* Register vips_shutdown(). This may well not get called and many
 	 * platforms don't support it anyway.
 	 */
@@ -368,6 +381,10 @@ vips_leak( void )
 	fprintf( stderr, "%s", vips_buf_all( &buf ) );
 
 	vips__type_leak();
+
+#ifdef DEBUG
+#endif /*DEBUG*/
+	vips_buffer_dump_all();
 }
 
 /**
@@ -418,10 +435,16 @@ vips_shutdown( void )
 		vips__thread_gate_stop( "init: main" ); 
 }
 
+	vips__render_shutdown();
+
 	vips_thread_shutdown();
 
 	vips__thread_profile_stop();
 
+#ifdef HAVE_GSF
+	gsf_shutdown(); 
+#endif /*HAVE_GSF*/
+
 	/* In dev releases, always show leaks. But not more than once, it's
 	 * annoying.
 	 */
@@ -488,7 +511,7 @@ static GOptionEntry option_entries[] = {
 	{ "vips-fatal", 0, G_OPTION_FLAG_HIDDEN | G_OPTION_FLAG_NO_ARG, 
 		G_OPTION_ARG_CALLBACK, (gpointer) &vips_set_fatal_cb, 
 		N_( "abort on first error or warning" ), NULL },
-	{ "vips-concurrency", 'c', 0, 
+	{ "vips-concurrency", 0, 0, 
 		G_OPTION_ARG_INT, &vips__concurrency, 
 		N_( "evaluate with N concurrent threads" ), "N" },
 	{ "vips-tile-width", 0, G_OPTION_FLAG_HIDDEN, 
@@ -796,7 +819,7 @@ const char *
 vips_guess_prefix( const char *argv0, const char *env_name )
 {
         const char *prefix;
-        const char *p;
+        char *basename;
         char name[VIPS_PATH_MAX];
 
 	/* Already set?
@@ -811,18 +834,20 @@ vips_guess_prefix( const char *argv0, const char *env_name )
 
 	/* Get the program name from argv0.
 	 */
-	p = vips_skip_dir( argv0 );
+	basename = g_path_get_basename( argv0 );
 
 	/* Add the exe suffix, if it's missing.
 	 */
 	if( strlen( VIPS_EXEEXT ) > 0 ) {
 		const char *olds[] = { VIPS_EXEEXT };
 
-		vips__change_suffix( p, name, 
+		vips__change_suffix( basename, name, 
 			VIPS_PATH_MAX, VIPS_EXEEXT, olds, 1 );
 	}
 	else
-		vips_strncpy( name, p, VIPS_PATH_MAX );
+		vips_strncpy( name, basename, VIPS_PATH_MAX );
+
+	g_free( basename ); 
 
 #ifdef DEBUG
 	printf( "vips_guess_prefix: argv0 = %s\n", argv0 );
