@@ -52,6 +52,8 @@
 
 #include <gobject/gvaluecollector.h>
 
+#include "vipsmarshal.h"
+
 /**
  * SECTION: object
  * @short_description: the VIPS base object class
@@ -79,9 +81,39 @@
  *
  */
 
+/** 
+ * VipsArgumentFlags:
+ * @VIPS_ARGUMENT_NONE: no flags
+ * @VIPS_ARGUMENT_REQUIRED: must be set in the constructor
+ * @VIPS_ARGUMENT_CONSTRUCT: can only be set in the constructor
+ * @VIPS_ARGUMENT_SET_ONCE: can only be set once
+ * @VIPS_ARGUMENT_SET_ALWAYS: don't do use-before-set checks
+ * @VIPS_ARGUMENT_INPUT: is an input argument (one we depend on)
+ * @VIPS_ARGUMENT_OUTPUT: is an output argument (depends on us)
+ * @VIPS_ARGUMENT_DEPRECATED: just there for back-compat, hide 
+ *
+ * Flags we associate with each object argument.
+ *
+ * Have separate input & output flags. Both set is an error; neither set is OK.
+ *
+ * Input gobjects are automatically reffed, output gobjects automatically ref
+ * us. We also automatically watch for "destroy" and unlink.
+ *
+ * @VIPS_ARGUMENT_SET_ALWAYS is handy for arguments which are set from C. For
+ * example, VipsImage::width is a property that gives access to the Xsize
+ * member of struct _VipsImage. We default its 'assigned' to TRUE
+ * since the field is always set directly by C.
+ *
+ * @VIPS_ARGUMENT_DEPRECATED arguments are not shown in help text, are not
+ * looked for if required, are not checked for "have-been-set". You can
+ * deprecate a required argument, but you must obviously add a new required
+ * argument if you do.
+ */
+
 /* Our signals. 
  */
 enum {
+	SIG_POSTBUILD,		
 	SIG_PRECLOSE,		
 	SIG_CLOSE,		
 	SIG_POSTCLOSE,		
@@ -97,8 +129,30 @@ static guint vips_object_signals[SIG_LAST] = { 0 };
 
 int _vips__argument_id = 1;
 
+/* Keep a cache of nickname -> GType lookups.
+ */
+static GHashTable *vips__object_nickname_table = NULL;
+
 G_DEFINE_ABSTRACT_TYPE( VipsObject, vips_object, G_TYPE_OBJECT );
 
+/* Don't call this directly, see vips_object_build().
+ */
+static int
+vips_object_postbuild( VipsObject *object )
+{
+	int result;
+
+#ifdef DEBUG
+	printf( "vips_object_postbuild: " );
+	vips_object_print_name( object );
+	printf( "\n" );
+#endif /*DEBUG*/
+
+	g_signal_emit( object, vips_object_signals[SIG_POSTBUILD], 0, &result );
+
+	return( result ); 
+}
+
 void
 vips_object_preclose( VipsObject *object )
 {
@@ -217,6 +271,11 @@ vips_object_build( VipsObject *object )
 	 */
 	object->constructed = TRUE;
 
+	/* Only postbuild on success.
+	 */
+	if( !result )
+		result = vips_object_postbuild( object );
+
 	return( result );
 }
 
@@ -333,16 +392,37 @@ vips_object_rewind( VipsObject *object )
 /* Extra stuff we track for properties to do our argument handling.
  */
 
+static void
+vips_argument_instance_detach( VipsArgumentInstance *argument_instance )
+{
+	VipsObject *object = argument_instance->object;
+	VipsArgumentClass *argument_class = argument_instance->argument_class;
+	GObject *member = G_STRUCT_MEMBER( GObject *, object,
+		argument_class->offset );
+
+	if( argument_instance->close_id ) {
+		if( g_signal_handler_is_connected( member,
+			argument_instance->close_id ) )
+			g_signal_handler_disconnect( member,
+				argument_instance->close_id );
+		argument_instance->close_id = 0;
+	}
+
+	if( argument_instance->invalidate_id ) {
+		if( g_signal_handler_is_connected( member,
+			argument_instance->invalidate_id ) )
+			g_signal_handler_disconnect( member,
+				argument_instance->invalidate_id );
+		argument_instance->invalidate_id = 0;
+	}
+}
+
 /* Free a VipsArgumentInstance ... VipsArgumentClass can just be g_free()d.
  */
 static void
 vips_argument_instance_free( VipsArgumentInstance *argument_instance )
 {
-	if( argument_instance->close_id ) {
-		g_signal_handler_disconnect( argument_instance->object,
-			argument_instance->close_id );
-		argument_instance->close_id = 0;
-	}
+	vips_argument_instance_detach( argument_instance );
 	g_free( argument_instance );
 }
 
@@ -439,6 +519,40 @@ vips_argument_class_map( VipsObjectClass *object_class,
 	return( NULL );
 }
 
+/* Does an vipsargument need an argument to write to? For example, an image
+ * output needs a filename, a double output just prints.
+ */
+gboolean
+vips_argument_class_needsstring( VipsArgumentClass *argument_class )
+{
+	GParamSpec *pspec = ((VipsArgument *) argument_class)->pspec;
+
+	GType otype;
+	VipsObjectClass *oclass;
+
+	if( G_IS_PARAM_SPEC_BOOLEAN( pspec ) ) 
+		/* Bools, input or output, don't need args.
+		 */
+		return( FALSE );
+
+	if( argument_class->flags & VIPS_ARGUMENT_INPUT ) 
+		/* All other inputs need something.
+		 */
+		return( TRUE );
+
+	/* Just output objects.
+	 */
+
+	if( (otype = G_PARAM_SPEC_VALUE_TYPE( pspec )) &&
+		g_type_is_a( otype, VIPS_TYPE_OBJECT ) &&
+		(oclass = g_type_class_ref( otype )) )
+		/* For now, only vipsobject subclasses can ask for args.
+		 */
+		return( oclass->output_needs_arg );
+	else
+		return( FALSE );
+}
+
 /* Create a VipsArgumentInstance for each installed argument property. Ideally
  * we'd do this during _init() but g_object_class_find_property() does not seem
  * to work then :-( so we have to delay it until first access. See
@@ -487,6 +601,7 @@ vips_argument_init( VipsObject *object )
 				argument_class->flags & 
 					VIPS_ARGUMENT_SET_ALWAYS;
 			argument_instance->close_id = 0;
+			argument_instance->invalidate_id = 0;
 
 			vips_argument_table_replace( object->argument_table, 
 				(VipsArgument *) argument_instance );
@@ -627,14 +742,14 @@ vips_object_get_argument_priority( VipsObject *object, const char *name )
 }
 
 static void
-vips_object_clear_member( VipsObject *object, GParamSpec *pspec, 
-	GObject **member )
+vips_object_clear_member( VipsArgumentInstance *argument_instance )
 {
-	VipsObjectClass *class = VIPS_OBJECT_GET_CLASS( object );
-	VipsArgumentClass *argument_class = (VipsArgumentClass *)
-		vips__argument_table_lookup( class->argument_table, pspec );
-	VipsArgumentInstance *argument_instance =
-		vips__argument_get_instance( argument_class, object );
+	VipsObject *object = argument_instance->object;
+	VipsArgumentClass *argument_class = argument_instance->argument_class;
+	GObject **member = &G_STRUCT_MEMBER( GObject *, object,
+		argument_class->offset );
+
+	vips_argument_instance_detach( argument_instance );
 
 	if( *member ) {
 		if( argument_class->flags & VIPS_ARGUMENT_INPUT ) {
@@ -661,17 +776,6 @@ vips_object_clear_member( VipsObject *object, GParamSpec *pspec,
 				G_OBJECT( object )->ref_count - 1 );
 #endif /*DEBUG_REF*/
 
-			/* The object reffed us. Stop listening link to the
-			 * object's "close" signal. We can come here from
-			 * object being closed, in which case the handler
-			 * will already have been disconnected for us.
-			 */
-			if( g_signal_handler_is_connected( object,
-				argument_instance->close_id ) )
-				g_signal_handler_disconnect( object,
-					argument_instance->close_id );
-			argument_instance->close_id = 0;
-
 			g_object_unref( object );
 		}
 
@@ -826,6 +930,18 @@ vips_object_finalize( GObject *gobject )
 	G_OBJECT_CLASS( vips_object_parent_class )->finalize( gobject );
 }
 
+static void
+vips_object_arg_invalidate( GObject *argument,
+	VipsArgumentInstance *argument_instance )
+{
+	/* Image @argument has signalled "invalidate" ... resignal on our
+	 * operation.
+	 */
+	if( VIPS_IS_OPERATION( argument_instance->object ) )
+		vips_operation_invalidate( 
+			VIPS_OPERATION( argument_instance->object ) ); 
+}
+
 static void
 vips_object_arg_close( GObject *argument,
 	VipsArgumentInstance *argument_instance )
@@ -853,10 +969,11 @@ vips__object_set_member( VipsObject *object, GParamSpec *pspec,
 		vips__argument_table_lookup( class->argument_table, pspec );
 	VipsArgumentInstance *argument_instance =
 		vips__argument_get_instance( argument_class, object );
+	GType otype = G_PARAM_SPEC_VALUE_TYPE( pspec );
 
 	g_assert( argument_instance );
 
-	vips_object_clear_member( object, pspec, member );
+	vips_object_clear_member( argument_instance );
 
 	g_assert( !*member );
 	*member = argument;
@@ -864,7 +981,7 @@ vips__object_set_member( VipsObject *object, GParamSpec *pspec,
 	if( *member ) {
 		if( argument_class->flags & VIPS_ARGUMENT_INPUT ) {
 #ifdef DEBUG_REF
-			printf( "vips_object_set_member: vips object: " );
+			printf( "vips__object_set_member: vips object: " );
 			vips_object_print_name( object );
 			printf( "  refers to gobject %s (%p)\n",
 				G_OBJECT_TYPE_NAME( *member ), *member );
@@ -878,7 +995,7 @@ vips__object_set_member( VipsObject *object, GParamSpec *pspec,
 		}
 		else if( argument_class->flags & VIPS_ARGUMENT_OUTPUT ) {
 #ifdef DEBUG_REF
-			printf( "vips_object_set_member: gobject %s (%p)\n",
+			printf( "vips__object_set_member: gobject %s (%p)\n",
 				G_OBJECT_TYPE_NAME( *member ), *member );
 			printf( "  refers to vips object: " );
 			vips_object_print_name( object );
@@ -889,10 +1006,23 @@ vips__object_set_member( VipsObject *object, GParamSpec *pspec,
 			/* The argument reffs us.
 			 */
 			g_object_ref( object );
+		}
+	}
 
-			/* FIXME ... could use a NULLing weakref
-			 */
+	if( *member &&
+		g_type_is_a( otype, VIPS_TYPE_IMAGE ) ) { 
+		if( argument_class->flags & VIPS_ARGUMENT_INPUT ) {
+			g_assert( !argument_instance->invalidate_id );
+
+			argument_instance->invalidate_id =
+				g_signal_connect( *member, "invalidate",
+					G_CALLBACK( 
+						vips_object_arg_invalidate ),
+					argument_instance );
+		}
+		else if( argument_class->flags & VIPS_ARGUMENT_OUTPUT ) {
 			g_assert( !argument_instance->close_id );
+
 			argument_instance->close_id =
 				g_signal_connect( *member, "close",
 					G_CALLBACK( vips_object_arg_close ),
@@ -1005,7 +1135,7 @@ vips_object_set_property( GObject *gobject,
 		GObject **member = &G_STRUCT_MEMBER( GObject *, object,
 			argument_class->offset );
 
-		vips__object_set_member( object, pspec, member, 
+		vips__object_set_member( object, pspec, member,
 			g_value_get_object( value ) );
 	}
 	else if( G_IS_PARAM_SPEC_INT( pspec ) ) {
@@ -1218,6 +1348,20 @@ vips_object_real_build( VipsObject *object )
 	return( result );
 }
 
+static int
+vips_object_real_postbuild( VipsObject *object )
+{
+#ifdef DEBUG
+	printf( "vips_object_real_postbuild: " ); 
+	vips_object_print_name( object );
+	printf( "\n" );
+#endif /*DEBUG*/
+
+	g_assert( object->constructed ); 
+
+	return( 0 ); 
+}
+
 static void
 vips_object_real_summary_class( VipsObjectClass *class, VipsBuf *buf )
 {
@@ -1317,6 +1461,7 @@ vips_object_class_init( VipsObjectClass *class )
 	gobject_class->get_property = vips_object_get_property;
 
 	class->build = vips_object_real_build;
+	class->postbuild = vips_object_real_postbuild;
 	class->summary_class = vips_object_real_summary_class;
 	class->summary = vips_object_real_summary;
 	class->dump = vips_object_real_dump;
@@ -1352,6 +1497,13 @@ vips_object_class_init( VipsObjectClass *class )
 		G_STRUCT_OFFSET( VipsObject, description ), 
 		"" );
 
+	vips_object_signals[SIG_POSTBUILD] = g_signal_new( "postbuild",
+		G_TYPE_FROM_CLASS( class ),
+		G_SIGNAL_RUN_LAST,
+		G_STRUCT_OFFSET( VipsObjectClass, postbuild ), 
+		NULL, NULL,
+		vips_INT__VOID,
+		G_TYPE_INT, 0 );
 	vips_object_signals[SIG_PRECLOSE] = g_signal_new( "preclose",
 		G_TYPE_FROM_CLASS( class ),
 		G_SIGNAL_RUN_LAST,
@@ -1557,8 +1709,7 @@ vips_object_set_argument_from_string( VipsObject *object,
 			return( -1 );
 		}
 
-		/* Read the filename. vips_foreign_load_options()
-		 * handles embedded options.
+		/* Read the filename. 
 		 */
 		if( flags & VIPS_OPERATION_SEQUENTIAL_UNBUFFERED ) 
 			access = VIPS_ACCESS_SEQUENTIAL_UNBUFFERED;
@@ -1567,7 +1718,7 @@ vips_object_set_argument_from_string( VipsObject *object,
 		else
 			access = VIPS_ACCESS_RANDOM; 
 
-		if( vips_foreign_load_options( value, &out, 
+		if( vips_foreign_load( value, &out, 
 			"access", access,
 			NULL ) )
 			return( -1 );
@@ -1648,7 +1799,7 @@ vips_object_set_argument_from_string( VipsObject *object,
 			return( -1 );
 		}
 
-		if( sscanf( value, "%Ld", &l ) != 1 ) {
+		if( sscanf( value, "%lld", &l ) != 1 ) {
 			vips_error( class->nickname,
 				_( "'%s' is not an integer" ), value );
 			return( -1 );
@@ -1731,10 +1882,8 @@ gboolean
 vips_object_argument_needsstring( VipsObject *object, const char *name )
 {
 	GParamSpec *pspec;
-	GType otype;
 	VipsArgumentClass *argument_class;
 	VipsArgumentInstance *argument_instance;
-	VipsObjectClass *oclass;
 
 #ifdef DEBUG
 	printf( "vips_object_argument_needsstring: %s\n", name );
@@ -1744,22 +1893,7 @@ vips_object_argument_needsstring( VipsObject *object, const char *name )
 		&pspec, &argument_class, &argument_instance ) )
 		return( -1 );
 
-	if( G_IS_PARAM_SPEC_BOOLEAN( pspec ) ) 
-		/* Bools, input or output, don't need args.
-		 */
-		return( FALSE );
-	else if( argument_class->flags & VIPS_ARGUMENT_INPUT ) 
-		/* All other inputs need something.
-		 */
-		return( TRUE );
-	if( (otype = G_PARAM_SPEC_VALUE_TYPE( pspec )) &&
-		g_type_is_a( otype, VIPS_TYPE_OBJECT ) &&
-		(oclass = g_type_class_ref( otype )) )
-		/* For now, only vipsobject subclasses can ask for args.
-		 */
-		return( oclass->output_needs_arg );
-	else
-		return( FALSE );
+	return( vips_argument_class_needsstring( argument_class ) ); 
 }
 
 static void
@@ -1808,10 +1942,9 @@ vips_object_get_argument_to_string( VipsObject *object,
 		VipsImage *in;
 
 		/* Pull out the image and write it.
-		 * vips_foreign_save_options() handles embedded options.
 		 */
 		g_object_get( object, name, &in, NULL );
-		if( vips_foreign_save_options( in, arg, NULL ) ) {
+		if( vips_foreign_save( in, arg, NULL ) ) {
 			g_object_unref( in );
 			return( -1 );
 		}
@@ -1956,7 +2089,7 @@ vips_object_set_valist( VipsObject *object, va_list ap )
  * Input arguments are given in-line, output arguments are given as pointers
  * to where the output value should be written.
  *
- * See also: vips_object_set_valist().
+ * See also: vips_object_set_valist(), vips_object_set_from_string(). 
  *
  * Returns: 0 on success, -1 on error
  */
@@ -1974,7 +2107,7 @@ vips_object_set( VipsObject *object, ... )
 }
 
 /* Set object args from a string. @p should be the initial left bracket and
- * there should be no tokens after the matching right bracket.
+ * there should be no tokens after the matching right bracket. @p is modified. 
  */
 static int
 vips_object_set_args( VipsObject *object, const char *p )
@@ -1992,10 +2125,18 @@ vips_object_set_args( VipsObject *object, const char *p )
 		string, VIPS_PATH_MAX )) )
 		return( -1 );
 
-	do {
-		if( !(p = vips__token_need( p, VIPS_TOKEN_STRING,
-			string, VIPS_PATH_MAX )) )
+	if( !(p = vips__token_must( p, &token, string, VIPS_PATH_MAX )) )
+		return( -1 );
+
+	for(;;) {
+		if( token == VIPS_TOKEN_RIGHT )
+			break;
+		if( token != VIPS_TOKEN_STRING ) {
+			vips_error( class->nickname,
+				_( "expected string or ), saw %s" ), 
+				vips_enum_nick( VIPS_TYPE_TOKEN, token ) );
 			return( -1 );
+		}
 
 		/* We have to look for a '=', ')' or a ',' to see if string is
 		 * a param name or a value.
@@ -2038,14 +2179,19 @@ vips_object_set_args( VipsObject *object, const char *p )
 			return( -1 );
 		}
 
-		/* Now must be a , or a ).
+		/* Now must be a , or a ). 
 		 */
-		if( token != VIPS_TOKEN_RIGHT && token != VIPS_TOKEN_COMMA ) {
+		if( token == VIPS_TOKEN_COMMA ) {
+			if( !(p = vips__token_must( p, &token, 
+				string, VIPS_PATH_MAX )) )
+				return( -1 );
+		}
+		else if( token != VIPS_TOKEN_RIGHT ) {
 			vips_error( class->nickname,
 				"%s", _( "not , or ) after parameter" ) );
 			return( -1 );
 		}
-	} while( token != VIPS_TOKEN_RIGHT );
+	}
 
 	if( (p = vips__token_get( p, &token, string, VIPS_PATH_MAX )) ) {
 		vips_error( class->nickname,
@@ -2056,6 +2202,44 @@ vips_object_set_args( VipsObject *object, const char *p )
 	return( 0 );
 }
 
+/**
+ * vips_object_set_from_string:
+ * @object: object to set arguments on
+ * @string: arguments as a string
+ *
+ * Set object arguments from a string. The string can be something like
+ * "a=12", or "a = 12, b = 13", or "fred". The string can optionally be
+ * enclosed in brackets. 
+ *
+ * You'd typically use this between creating the object and building it. 
+ *
+ * See also: vips_object_set(), vips_object_build(),
+ * vips_cache_operation_buildp(). 
+ *
+ * Returns: 0 on success, -1 on error
+ */
+int
+vips_object_set_from_string( VipsObject *object, const char *string )
+{
+	const char *q;
+	VipsToken token;
+	char buffer[VIPS_PATH_MAX];
+	char str[VIPS_PATH_MAX];
+
+	vips_strncpy( buffer, string, VIPS_PATH_MAX );
+
+	/* Does string start with a bracket? If it doesn't, enclose the whole
+	 * thing in [].
+	 */
+	if( !(q = vips__token_get( buffer, &token, str, VIPS_PATH_MAX )) ||
+		token != VIPS_TOKEN_LEFT )
+		vips_snprintf( buffer, VIPS_PATH_MAX, "[%s]", string );
+	else
+		vips_strncpy( buffer, string, VIPS_PATH_MAX );
+
+	return( vips_object_set_args( object, buffer ) ); 
+}
+
 VipsObject *
 vips_object_new_from_string( VipsObjectClass *object_class, const char *p )
 {
@@ -2079,7 +2263,7 @@ vips_object_new_from_string( VipsObjectClass *object_class, const char *p )
 	/* More tokens there? Set any other args.
 	 */
 	if( q && 
-		vips_object_set_args( object, q ) ) {
+		vips_object_set_from_string( object, q ) ) {
 		g_object_unref( object );
 		return( NULL );
 	}
@@ -2357,15 +2541,93 @@ vips_class_find( const char *basename, const char *nickname )
 	return( class );
 }
 
+/* What we store for each nickname. We can't just store the type with
+ * GINT_TO_POINTER() since GType is 64 bits on some platforms.
+ */
+typedef struct _NicknameGType {
+	const char *nickname;
+	GType type;
+	gboolean duplicate;
+} NicknameGType;
+
+static void *
+vips_class_add_hash( VipsObjectClass *class, GHashTable *table )
+{
+	GType type = G_OBJECT_CLASS_TYPE( class );
+	NicknameGType *hit;
+
+	hit = (NicknameGType *) 
+		g_hash_table_lookup( table, (void *) class->nickname );
+
+	/* If this is not a unique name, mark as a duplicate. In this case
+	 * we'll need to fall back to a search.
+	 */
+	if( hit ) 
+		hit->duplicate = TRUE;
+	else {
+		hit = g_new( NicknameGType, 1 );
+		hit->nickname = class->nickname;
+		hit->type = type;
+		hit->duplicate = FALSE;
+		g_hash_table_insert( table, (void *) hit->nickname, hit );
+	}
+
+	return( NULL ); 
+}
+
+static void *
+vips_class_build_hash( void )
+{
+	GHashTable *table;
+	GType base;
+
+	table = g_hash_table_new( g_str_hash, g_str_equal );
+
+	if( !(base = g_type_from_name( "VipsObject" )) )
+		return( NULL );
+	vips_class_map_all( base, 
+		(VipsClassMapFn) vips_class_add_hash, (void *) table );
+
+	return( table ); 
+}
+
 GType
 vips_type_find( const char *basename, const char *nickname )
 {
-	VipsObjectClass *class;
+	static GOnce once = G_ONCE_INIT;
 
-	if( !(class = vips_class_find( basename, nickname )) )
+	const char *classname = basename ? basename : "VipsObject";
+
+	NicknameGType *hit;
+	GType base;
+	GType type;
+
+	vips__object_nickname_table = (GHashTable *) g_once( &once, 
+		(GThreadFunc) vips_class_build_hash, NULL ); 
+
+	hit = (NicknameGType *) 
+		g_hash_table_lookup( vips__object_nickname_table, 
+			(void *) nickname );
+
+	/* We must only search below basename ... check that the cache hit is
+	 * in the right part of the tree.
+	 */
+	if( !(base = g_type_from_name( classname )) )
 		return( 0 );
+	if( hit &&
+		!hit->duplicate &&
+		g_type_is_a( hit->type, base ) ) 
+		type = hit->type;
+	else {
+		VipsObjectClass *class;
 
-	return( G_OBJECT_CLASS_TYPE( class ) );
+		if( !(class = vips_class_find( basename, nickname )) )
+			return( 0 );
+
+		type = G_OBJECT_CLASS_TYPE( class );
+	}
+
+	return( type );
 }
 
 /* The vips_object_local() macro uses this as its callback.
