@@ -801,7 +801,8 @@ transform_g_string_array_image( const GValue *src_value, GValue *dest_value )
 	str = g_value_dup_string( src_value );
 
 	for( i = 0, p = str; (q = vips_break_token( p, " " )); i++, p = q )
-		if( !(array[i] = G_OBJECT( vips_image_new_from_file( p ) )) ) {
+		if( !(array[i] = G_OBJECT( vips_image_new_from_file( p, 
+			NULL ) )) ) {
 			/* Set the dest to length zero to indicate error.
 			 */
 			vips_value_set_array_object( dest_value, 0 );
@@ -1275,6 +1276,50 @@ vips_value_set_array_double( GValue *value, const double *array, int n )
 	return( 0 );
 }
 
+/** 
+ * vips_value_get_array_image:
+ * @value: %GValue to get from
+ * @n: (allow-none): return the number of elements here, optionally
+ *
+ * Return the start of the array of images held by @value.
+ * optionally return the number of elements in @n.
+ *
+ * See also: vips_value_set_array_image().
+ *
+ * Returns: (transfer none): The array address.
+ */
+VipsImage **
+vips_value_get_array_image( const GValue *value, int *n )
+{
+	return( vips_value_get_array( value, n, NULL, NULL ) );
+}
+
+/** 
+ * vips_value_set_array_image:
+ * @value: (out): %GValue to get from
+ * @array: (array length=n): array of images
+ * @n: the number of elements 
+ *
+ * Set @value to hold a copy of @array. Pass in the array length in @n. 
+ *
+ * See also: vips_array_image_get().
+ *
+ * Returns: 0 on success, -1 otherwise.
+ */
+int
+vips_value_set_array_image( GValue *value, VipsImage **array, int n )
+{
+	VipsImage **array_copy;
+
+	g_value_init( value, VIPS_TYPE_ARRAY_IMAGE );
+	vips_value_set_array( value, n, VIPS_TYPE_ARRAY_IMAGE, 
+		sizeof( VipsImage * ) );
+	array_copy = vips_value_get_array_image( value, NULL );
+	memcpy( array_copy, array, n * sizeof( VipsImage * ) );
+
+	return( 0 );
+}
+
 /** 
  * vips_value_get_array_object: (skip)
  * @value: %GValue to get from
