@@ -112,7 +112,6 @@
 
 /*
 #define DEBUG
-#define DEBUG_GEOMETRY
  */
 
 #ifdef HAVE_CONFIG_H
@@ -384,9 +383,7 @@ vips_affine_build( VipsObject *object )
 	VipsObjectClass *class = VIPS_OBJECT_GET_CLASS( object );
 	VipsResample *resample = VIPS_RESAMPLE( object );
 	VipsAffine *affine = (VipsAffine *) object;
-
-	VipsImage **t = (VipsImage **) 
-		vips_object_local_array( object, 4 );
+	VipsImage **t = (VipsImage **) vips_object_local_array( object, 4 );
 
 	VipsImage *in;
 	VipsDemandStyle hint; 
@@ -469,18 +466,9 @@ vips_affine_build( VipsObject *object )
 		return( -1 );
 	}
 
-	/* Unpack for processing.
-	 */
-	if( in->Coding == VIPS_CODING_LABQ ) {
-		if( vips_LabQ2LabS( in, &t[0], NULL ) )
-			return( -1 );
-		in = t[0];
-	} 
-	else if( in->Coding == VIPS_CODING_RAD ) {
-		if( vips_rad2float( in, &t[1], NULL ) )
-			return( -1 );
-		in = t[1];
-	}
+	if( vips_image_decode( in, &t[0] ) )
+		return( -1 );
+	in = t[0];
 
 	/* Add new pixels around the input so we can interpolate at the edges.
 	 */
@@ -506,6 +494,15 @@ vips_affine_build( VipsObject *object )
 	resample->out->Xsize = affine->trn.oarea.width;
 	resample->out->Ysize = affine->trn.oarea.height;
 
+#ifdef DEBUG
+	printf( "vips_affine_build: transform: "
+		"a = %g, b = %g, c = %g, d = %g\n",
+		affine->trn.a, 
+		affine->trn.b, 
+		affine->trn.c, 
+		affine->trn.d );  
+#endif /*DEBUG*/
+
 	/* Generate!
 	 */
 	if( vips_image_generate( resample->out, 
